from collections import Counter
import mysql.connector
from datetime import datetime, timedelta
from flask import jsonify
import re
import os

def count_words(site_id, start_date, end_date, program):
    start_date, end_date = validate_and_format_dates(start_date, end_date)

    table_posts = f"{site_id}posts"
    table_postmeta = f"{site_id}postmeta"
    table_terms = f"{site_id}terms"
    table_term_tax = f"{site_id}term_taxonomy"

    host = os.environ.get("WP_DB_HOST")
    user = os.environ.get("WP_DB_USER")
    password = os.environ.get("WP_DB_PASSWORD")
    database = os.environ.get("WP_DB_DATABASE")
    port = os.environ.get("WP_DB_PORT")

    result = None

    mydb = mysql.connector.connect(
            host=host,
            user=user,
            password=password,
            database=database,
            port=port
            )

    try:
        terms_cursor = mydb.cursor(dictionary=True)

        sql_terms = f"""
            SELECT 
                t.name AS label,
                t.slug AS term,
                t.term_id AS id,
                (SELECT slug FROM {table_terms} WHERE term_id = tt.parent) AS parent_slug,
                (SELECT name FROM {table_terms} WHERE term_id = tt.parent) AS parent_label,
                (SELECT slug FROM {table_terms} WHERE term_id = 
                    (SELECT parent FROM {table_term_tax} WHERE term_id = tt.parent)
                ) AS gparent_slug
            FROM
                {table_terms} t
            INNER JOIN
                {table_term_tax} tt
            ON t.term_id = tt.term_id
            WHERE
                tt.taxonomy = "nuvem_palavras"
            ORDER BY
                gparent_slug, term;
        """

        terms_cursor.execute(sql_terms)
        terms_result = terms_cursor.fetchall()
        
        program_data = [
            term
            for term in terms_result
            if  term["gparent_slug"] == program 
        ]

        terms_list = [term["term"] for term in program_data if term["gparent_slug"] == program ]
        filters = re.findall(r'\w+', " ".join(row for row in terms_list))

#        sql_filter = f"""
#            SELECT t.slug AS word, t.name AS label, tt.parent AS parent_id, t.term_id AS id
#            FROM {table_terms} t
#            JOIN {table_term_tax} tt ON t.term_id = tt.term_taxonomy_id
#            WHERE tt.taxonomy = 'nuvem_palavras'
#        """
#         
#         filter_words.execute(sql_filter)
#         filter_words_results = filter_words.fetchall()
#         programs = [dicionario for dicionario in filter_words_results if dicionario.get('parent_id') == 0]
#         program = next((p for p in programs if p.get('word') == program), None)
#         word_themes = [
#             dic for dic in filter_words_results
#             if dic.get('parent_id') == program['id'] if program
#         ]
#         terms = [
#             dic for dic in filter_words_results
#             if dic.get('parent_id') in (theme['id'] for theme in word_themes)
#         ]
#         filters = re.findall(r'\w+', " ".join(row['word'] for row in terms).lower())

        msgs_cursor = mydb.cursor()

        sql_msgs = f"""
             SELECT pm.meta_value as message 
             FROM {table_posts} p
             LEFT JOIN {table_postmeta} pm
             ON p.ID = pm.post_id
             WHERE pm.meta_key = "mensagem"
             AND p.post_date BETWEEN "{start_date}" AND "{end_date}"
         """
        
#         sql_msgs = f"""
#              SELECT pm.meta_value as message 
#              FROM wp_6_posts p
#              LEFT JOIN wp_6_postmeta pm
#              ON p.ID = pm.post_id
#              WHERE pm.meta_key = "mensagem"
#              AND p.post_date BETWEEN '2000-01-01 00:00:00' AND '2024-09-01 00:00:00'
#          """

        msgs_cursor.execute(sql_msgs)
        msgs_result = msgs_cursor.fetchall()
        msg_words = re.findall(r'\w{5,}', " ".join(row[0] for row in msgs_result).lower())
        count = Counter(word for word in msg_words if word in filters) 
        prog_data_map = {item['term']: item for item in program_data}

        data_count = {
            key: {
                "label": prog_data_map[key]["label"],
                "theme_label": prog_data_map[key]["parent_label"],
                "theme_slug": prog_data_map[key]["parent_slug"]
            } for key in count if key in prog_data_map
        }
        
        for key, value in data_count.items():
            value['count'] = count[key]

        result = [value for key, value in data_count.items()]
        result.append([])

        hist = []
        for key in range(len(result) - 1):
            slug = result[key]["theme_slug"]  
            if slug not in hist:
                hist.append(slug)
                result[-1].append({result[key]["theme_slug"]: result[key]["theme_label"]})

    except mysql.connector.Error as error:
        print("Erro ao conectar ao banco de dados:", error)
        return jsonify({"error": error})
    finally:
        if mydb:
            mydb.close()

    return jsonify(result)

def validate_and_format_dates(start_date, end_date):
    if start_date:
        start_date = validate_date_format(start_date)
    else:
        if end_date:
            end_date = validate_date_format(end_date)
            start_date = end_date.replace(hour=0, minute=0, second=0)
        else:
            start_date = datetime.now().replace(hour=0, minute=0, second=0)
            end_date = datetime.now().replace(hour=23, minute=59, second=59)
    if end_date:
        end_date = validate_date_format(end_date)
    else:
        end_date = start_date.replace(hour=23, minute=59, second=59)
    return start_date, end_date

def validate_date_format(date_str):
    try:
        if isinstance(date_str, datetime):
            return date_str
        else:
            return datetime.strptime(date_str, "%Y-%m-%d %H:%M:%S")
    except ValueError:
        raise ValueError(f"Invalid date format. Please use 'YYYY-MM-DD hh:mm:ss")

